@@ -132,7 +132,8 @@ extern int yydebug;
     VAR_MAX_RETRY_TIME = 342,
     VAR_MIN_RETRY_TIME = 343,
     VAR_MULTI_MASTER_CHECK = 344,
-    VAR_MINIMAL_RESPONSES = 345
+    VAR_MINIMAL_RESPONSES = 345,
+    VAR_REFUSE_ANY = 346
   };
 #endif
 /* Tokens.  */
@@ -224,6 +225,7 @@ extern int yydebug;
 #define VAR_MIN_RETRY_TIME 343
 #define VAR_MULTI_MASTER_CHECK 344
 #define VAR_MINIMAL_RESPONSES 345
+#define VAR_REFUSE_ANY 346
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -234,7 +236,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 238 "configparser.h" /* yacc.c:1909  */
+#line 240 "configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
