@@ -545,7 +545,11 @@ remote_accept_callback(int fd, short event, void* arg)
 
 	/* perform the accept */
 	addrlen = sizeof(addr);
+#ifndef HAVE_ACCEPT4
 	newfd = accept(fd, (struct sockaddr*)&addr, &addrlen);
+#else
+	newfd = accept4(fd, (struct sockaddr*)&addr, &addrlen, SOCK_NONBLOCK);
+#endif
 	if(newfd == -1) {
 		if (    errno != EINTR
 			&& errno != EWOULDBLOCK
@@ -569,10 +573,13 @@ remote_accept_callback(int fd, short event, void* arg)
 		close(newfd);
 		return;
 	}
+
+#ifndef HAVE_ACCEPT4
 	if (fcntl(newfd, F_SETFL, O_NONBLOCK) == -1) {
 		log_msg(LOG_ERR, "fcntl failed: %s", strerror(errno));
 		goto close_exit;
 	}
+#endif
 
 	/* setup state to service the remote control command */
 	n = (struct rc_state*)calloc(1, sizeof(*n));
@@ -958,10 +965,17 @@ print_zonestatus(SSL* ssl, xfrd_state_type* xfrd, struct zone_options* zo)
 		if(nz->is_waiting) {
 			if(!ssl_printf(ssl, "	notify: \"waiting-for-fd\"\n"))
 				return 0;
-		} else if(nz->notify_send_enable) {
-			if(!ssl_printf(ssl, "	notify: \"sent try %d "
-				"to %s with serial %u\"\n", nz->notify_retry,
-				nz->notify_current->ip_address_spec,
+		} else if(nz->notify_send_enable || nz->notify_send6_enable) {
+			int i;
+			if(!ssl_printf(ssl, "	notify: \"send"))
+				return 0;
+			for(i=0; i<NOTIFY_CONCURRENT_MAX; i++) {
+				if(!nz->pkts[i].dest) continue;
+				if(!ssl_printf(ssl, " %s",
+					nz->pkts[i].dest->ip_address_spec))
+					return 0;
+			}
+			if(!ssl_printf(ssl, " with serial %u\"\n",
 				(unsigned)ntohl(nz->current_soa->serial)))
 				return 0;
 		}
@@ -985,6 +999,10 @@ print_zonestatus(SSL* ssl, xfrd_state_type* xfrd, struct zone_options* zo)
 		if(!print_soa_status(ssl, "notified-serial", &xz->soa_notified,
 			xz->soa_notified_acquired))
 			return 0;
+	} else if(xz->event_added) {
+		if(!ssl_printf(ssl, "\twait: \"%u sec between attempts\"\n",
+			(unsigned)xz->timeout.tv_sec))
+			return 0;
 	}
 
 	/* UDP */
